/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import ClayIcon from '@clayui/icon';
import ClayLayout from '@clayui/layout';
import ClayPopover from '@clayui/popover';
import PropTypes from 'prop-types';
import React, {useState} from 'react';

import useOnClickOutside from '../hooks/useOnClickOutside';
import UserIcon from './UserIcon';

function ReplyPopover({
	ariaLabel,
	contentHTML,
	href = '#',
	portraitURL,
	time,
	userId,
	username,
}) {
	const [openPopover, setOpenPopover] = useState(false);

	useOnClickOutside(
		['.lfr-discussion-reply-popover', '.lfr-discussion-parent-link'],
		() => setOpenPopover(false)
	);

	return (
		<ClayPopover
			alignPosition={Liferay.Browser.isMobile() ? 'top-left' : 'top'}
			className="lfr-discussion-reply-popover"
			header={
				<ClayLayout.ContentRow noGutters="x" padded>
					<ClayLayout.ContentCol>
						<UserIcon
							fullName={username}
							portraitURL={portraitURL}
							userId={userId}
						/>
					</ClayLayout.ContentCol>
					<ClayLayout.ContentCol expand>
						<div className="username">{username}</div>
						<div className="font-weight-normal text-secondary">
							{time}
						</div>
					</ClayLayout.ContentCol>
				</ClayLayout.ContentRow>
			}
			onShowChange={setOpenPopover}
			show={openPopover}
			trigger={
				<a
					aria-label={ariaLabel}
					className="lfr-discussion-parent-link"
					href={href}
					onClick={(event) => {
						event.preventDefault();
						setOpenPopover((open) => !open);
					}}
				>
					<ClayIcon
						className="inline-item inline-item-before"
						small="true"
						symbol="redo"
					/>
					{username}
				</a>
			}
		>
			<div dangerouslySetInnerHTML={{__html: contentHTML}}></div>
		</ClayPopover>
	);
}

ReplyPopover.propTypes = {
	ariaLabel: PropTypes.string,
	contentHTML: PropTypes.string.isRequired,
	href: PropTypes.string,
	time: PropTypes.string.isRequired,
	username: PropTypes.string.isRequired,
};

export default ReplyPopover;
